/*
 * This application demonstrates how to use the functions provided by the StopWatch utility
 *
 *
 */

#include "predef.h"
#include <stdio.h>
#include <ctype.h>

#ifdef MOD5213
#include <basictypes.h>
#include <serialirq.h>
#include <system.h>
#include <constants.h>
#include <ucos.h>
#include <SerialUpdate.h>
#include <smarttrap.h>
#include <gdbstub.h>

#else
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <NetworkDebug.h>
#endif

#if( ( defined MOD5270 ) || ( defined MOD5282 ) || (defined MOD5234) || (defined MOD5213)  )
#include <pins.h>
#endif

#include <utils.h>
#include <stdlib.h>
#include "StopWatch.h"
#include <taskmon.h>

extern "C"
{
void UserMain( void * pd );
}

const char * AppName = "Timer_Example";

DWORD LEDValue = 0;
double TotalTime = 0;
char Buffer[1000];

void IncrementLEDs( int Timer );


void SetGPIOtoLow();          // Timer pin low in GPIO for driver callback
void SetGPIOtoHigh();         // Timer pin high in GPIO for driver callback
void SetGPIOtoDMATimerOut();  // Timer pin set to timer out function for driver callback


void UserMain( void * pd )
{

#ifdef MOD5213
   SimpleUart(0,SystemBaud);
   assign_stdio(0);
   EnableSerialUpdate();
#else
   InitializeStack();
   if( EthernetIP == 0 )
      GetDHCPAddress();
   EnableAutoUpdate();
   EnableTaskMonitor();
#endif

   OSChangePrio( MAIN_PRIO );

   iprintf( "%s application started\n", AppName );


   PWM_GPIO_PIN_LOW_CALLBACK = SetGPIOtoLow;          // Points to user function that sets DMA Timer pin high in GPIO for PWM generation
   PWM_GPIO_PIN_HIGH_CALLBACK = SetGPIOtoHigh;         // Points to user function that sets DMA Timer pin low in GPIO for PWM generation
   DMA_PIN_TIMER_FUNC_CALLBACK = SetGPIOtoDMATimerOut;  // Points to user function that sets DMA Timer pin to DMA Timer Output function for PWM and Clock generation

   while( 1 )
   {
      iprintf( "\r\n1) Use DMA Timer to time an operation\r\n" );
      iprintf( "2) Use DMA Timer to repeatedly trigger an interrupt routine\r\n" );
      iprintf( "3) Use DMA Timer to generate a precise OSTimeDly()\r\n" );
      iprintf( "4) Use DMA Timer to generate a precise polling delay\r\n" );
      iprintf( "5) Use DMA Timer to generate a Clock output\r\n" );
      iprintf( "6) Use DMA Timer to generate a PWM output\r\n" );
      iprintf( "Select the operation to test :" );
      gets( Buffer );
      iprintf( "\r\n" );
      switch( Buffer[0] )
      {
         case '1':
         {
            StopWatchInit();
            iprintf( "Press enter to start StopWatch... \r\n" );
            gets( Buffer );
            StopWatchStart();
            iprintf( "Press enter to stop StopWatch... \r\n" );
            gets( Buffer );
            TotalTime = StopWatchReadTime();
            printf( "The total time elapsed was %g seconds\r\n", TotalTime );
            StopWatchStopClear();
            break;
         }
         case '2':
         {
            StopWatch_Interrupt_Function = IncrementLEDs; // Point timer interrupt call-back to LED function
            iprintf( "Input number of seconds (floating point) between LED increments:" );
            gets( Buffer );
            StopWatchInit( 1, strtod( Buffer, NULL ) );
            StopWatchStart();
            iprintf( "\r\nPress enter to disable incrementing LEDs\r\n" );
            gets( Buffer );
            StopWatchStopClear();
            StopWatch_Interrupt_Function = NULL;
            LEDValue = 0;
            putleds( 0 );
            break;
         }
         case '3':
         {
            iprintf( "Input number of seconds (floating point) for precise OSTimeDly():" );
            gets( Buffer );
            DWORD StartingTick = TimeTick;
            StopWatchDelay( strtod( Buffer, NULL ) );
            iprintf( "\r\nDelay finished in %d system ticks\r\n", ( TimeTick - StartingTick ) );
            break;
         }
         case '4':
         {
            iprintf( "Input number of seconds (floating point) for precise polling time delay:" );
            gets( Buffer );
            double DelayTime = strtod( Buffer, NULL );
            DWORD StartingTick = TimeTick;
            StopWatchPollingDelay( DelayTime );
            iprintf( "\r\nDelay finished in %d system ticks\r\n", ( TimeTick - StartingTick ) );
            break;
         }
         case '5':
         {
            //--------------------------------------------------
            // NOTE - You must manually set the correct timer output pin for the
            //        proper timer output function for the clock generator to work
            //--------------------------------------------------

            iprintf( "Input the Frequency (floating point) to generate a clock on timer 1 out:" );
            gets( Buffer );
            StopWatchClockGenerator( strtod( Buffer, NULL ) );
            iprintf( "\r\n" );
            break;
         }
         case '6':
         {
            //--------------------------------------------------
            // NOTE - You must manually set the correct timer output pin for the
            //        proper timer output function for the PWM to work
            //--------------------------------------------------

            iprintf( "Input the Period (float Hz) to generate a PWM on timer 1 out:" );
            gets( Buffer );
            double Period =  strtod( Buffer, NULL );
            iprintf( "\r\nInput the Duty Cycle (float Percent) to generate a PWM on timer 1 out:" );
            gets( Buffer );
            double DutyCycle =  strtod( Buffer, NULL );
            StopWatchPWMGenerator( Period, DutyCycle );
            iprintf( "\r\n" );
            break;
         }
         default:
            break;
      }
   }
}

// A simple funtion that increments the LEDs displayed on the dev-board
void IncrementLEDs( int Timer )
{
   // The Timer parameter is not used here since we do not need to determine which timer interrupted
   LEDValue++;
   putleds( LEDValue );
}


void SetGPIOtoDMATimerOut()
{
#if( ( defined MOD5270 ) || ( defined MOD5282 ) )
            J2[34].function( PINJ2_34_DTOUT1 );
#elif(defined MOD5234)
            J2[34].function( PINJ2_34_DT1OUT );
#elif(defined MOD5213)
            Pins[23].function( PIN23_DTOUT1 );
#endif
}

void SetGPIOtoHigh()
{
   #if( ( defined MOD5270 ) || ( defined MOD5282 ) || ( defined MOD5234 ) )
               J2[34] = 1;
               J2[34].function( PIN_GPIO );
   #elif(defined MOD5213)
               Pins[23] = 1;
               Pins[23].function( pinx_GPIO );
   #endif
}

void SetGPIOtoLow()
{
#if( ( defined MOD5270 ) || ( defined MOD5282 ) || ( defined MOD5234 ) )
            J2[34] = 0;
            J2[34].function( PIN_GPIO );
#elif(defined MOD5213)
            Pins[23] = 0;
            Pins[23].function( pinx_GPIO );
#endif
}
